#!/usr/bin/env python

"""Module containing the MemProtMDSim class and the command line interface."""
import argparse
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_io.api.common import *


class MemProtMDSim(BiobbObject):
    """
    | biobb_io MemProtMDSim
    | This class is a wrapper of the MemProtMD to download a simulation using its REST API.
    | Wrapper for the `MemProtMD DB REST API <http://memprotmd.bioch.ox.ac.uk/>`_ to download a simulation.

    Args:
        output_simulation (str): Path to the output simulation in a ZIP file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_sim.zip>`_. Accepted formats: zip (edam:format_3987).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB code.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.memprotmd_sim import memprotmd_sim
            prop = { 
                'pdb_code': '2VGB' 
            }
            memprotmd_sim(output_simulation='/path/to/newSimulation.zip', 
                        properties=prop)

    Info:
        * wrapped_software:
            * name: MemProtMD DB
            * license: Creative Commons
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_simulation, 
                properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = { 
            "out": { "output_simulation": output_simulation } 
        }

        # Properties specific for BB
        self.pdb_code = properties.get('pdb_code', None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """ Checks all the input/output paths and parameters """
        self.output_simulation = check_output_path(self.io_dict["out"]["output_simulation"], "output_simulation", False, out_log, self.__class__.__name__)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`MemProtMDSim <api.memprotmd_sim.MemProtMDSim>` api.memprotmd_sim.MemProtMDSim object."""
        
        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart(): return 0
        #self.stage_files()

        check_mandatory_property(self.pdb_code, 'pdb_code', self.out_log, self.__class__.__name__)

        # get simulation files and save to output
        json_string = get_memprotmd_sim(self.pdb_code, self.output_simulation, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0

def memprotmd_sim(output_simulation: str, properties: dict = None, **kwargs) -> int:
    """Execute the :class:`MemProtMDSim <api.memprotmd_sim.MemProtMDSim>` class and
    execute the :meth:`launch() <api.memprotmd_sim.MemProtMDSim.launch>` method."""

    return MemProtMDSim(output_simulation=output_simulation,
                    properties=properties, **kwargs).launch()

def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="Wrapper for the MemProtMD DB REST API (http://memprotmd.bioch.ox.ac.uk/) to download a simulation.", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('-c', '--config', required=False, help="This file can be a YAML file, JSON file or JSON string")

    # Specific args of each building block
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('-o', '--output_simulation', required=True, help="Path to the output simulation in a ZIP file. Accepted formats: zip.")

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    memprotmd_sim(output_simulation=args.output_simulation, 
                    properties=properties)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""Module containing the IdealSdf class and the command line interface."""
import argparse
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_io.api.common import *


class IdealSdf(BiobbObject):
    """
    | biobb_io IdealSdf
    | This class is a wrapper for downloading an ideal SDF ligand from the Protein Data Bank.
    | Wrapper for the `Protein Data Bank in Europe <https://www.ebi.ac.uk/pdbe/>`_ and the `Protein Data Bank <https://www.rcsb.org/>`_ for downloading a single ideal SDF ligand.

    Args:
        output_sdf_path (str): Path to the output SDF file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/ref_output.sdf>`_. Accepted formats: sdf (edam:format_3814).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **ligand_code** (*str*) - (None) RSCB PDB ligand code.
            * **api_id** (*str*) - ("pdbe") Identifier of the PDB REST API from which the SDF structure will be downloaded. Values: pdbe (`PDB in Europe REST API <https://www.ebi.ac.uk/pdbe/pdbe-rest-api>`_), pdb (`RCSB PDB REST API <https://data.rcsb.org/>`_).
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.ideal_sdf import ideal_sdf
            prop = { 
                'ligand_code': 'HYZ', 
                'api_id': 'pdbe' 
            }
            ideal_sdf(output_sdf_path='/path/to/newStructure.sdf', 
                        properties=prop)

    Info:
        * wrapped_software:
            * name: Protein Data Bank
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_sdf_path, 
                properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = { 
            "out": { "output_sdf_path": output_sdf_path } 
        }

        # Properties specific for BB
        self.api_id = properties.get('api_id', 'pdbe')
        self.ligand_code = properties.get('ligand_code', None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """ Checks all the input/output paths and parameters """
        self.output_sdf_path = check_output_path(self.io_dict["out"]["output_sdf_path"], "output_sdf_path", False, out_log, self.__class__.__name__)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`IdealSdf <api.ideal_sdf.IdealSdf>` api.ideal_sdf.IdealSdf object."""
        
        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart(): return 0
        #self.stage_files()

        check_mandatory_property(self.ligand_code, 'ligand_code', self.out_log, self.__class__.__name__)

        self.ligand_code = self.ligand_code.strip()

        # Downloading PDB file
        sdf_string = download_ideal_sdf(self.ligand_code, self.api_id, self.out_log, self.global_log)
        write_sdf(sdf_string, self.output_sdf_path, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0

def ideal_sdf(output_sdf_path: str, properties: dict = None, **kwargs) -> int:
    """Execute the :class:`IdealSdf <api.ideal_sdf.IdealSdf>` class and
    execute the :meth:`launch() <api.ideal_sdf.IdealSdf.launch>` method."""

    return IdealSdf(output_sdf_path=output_sdf_path,
                properties=properties, **kwargs).launch()

def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="This class is a wrapper for downloading an ideal SDF ligand from the Protein Data Bank.", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('-c', '--config', required=False, help="This file can be a YAML file, JSON file or JSON string")

    # Specific args of each building block
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('-o', '--output_sdf_path', required=True, help="Path to the output SDF file. Accepted formats: sdf.")

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    # Specific call of each building block
    ideal_sdf(output_sdf_path=args.output_sdf_path, 
        properties=properties)

if __name__ == '__main__':
    main()



""" Common functions for package api """
import os
import json
import requests
import re
import urllib.request
from pathlib import Path, PurePath
from biobb_common.tools import file_utils as fu

def check_output_path(path, argument, optional, out_log, classname):
    """ Checks output file """ 
    if optional and not path:
        return None
    if PurePath(path).parent and not Path(PurePath(path).parent).exists():
        fu.log(classname + ': Unexisting %s folder, exiting' % argument, out_log)
        raise SystemExit(classname + ': Unexisting %s folder' % argument)
    file_extension = PurePath(path).suffix
    if not is_valid_file(file_extension[1:], argument):
        fu.log(classname + ': Format %s in %s file is not compatible' % (file_extension[1:], argument), out_log)
        raise SystemExit(classname + ': Format %s in %s file is not compatible' % (file_extension[1:], argument))
    return path

def is_valid_file(ext, argument):
    """ Checks if file format is compatible """
    formats = {
        'output_sdf_path': ['sdf'],
        'output_pdb_path': ['pdb'],
        'output_simulations': ['json'],
        'output_simulation': ['zip'],
        'output_pdb_zip_path': ['zip'],
        'output_mutations_list_txt': ['txt'],
        'output_json_path': ['json'],
        'output_fasta_path': ['fasta'],
        'output_mmcif_path': ['mmcif', 'cif'],
    }
    return ext in formats[argument]

def download_pdb(pdb_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the pdb file.
    """

    if api_id == 'mmb':
        url = "https://mmb.irbbarcelona.org/api/pdb/" + pdb_code + "/coords/?"
    elif api_id == 'pdb':
        url = "https://files.rcsb.org/download/" + pdb_code + ".pdb"
    elif api_id == 'pdbe':
        url = "https://www.ebi.ac.uk/pdbe/entry-files/download/pdb" + pdb_code + ".ent"

    fu.log("Downloading %s from: %s" % (pdb_code, url), out_log, global_log)
    return requests.get(url).content.decode('utf-8')

def download_af(uniprot_code, out_log=None, global_log=None, classname=None):
    """
    Returns:
        String: Content of the pdb file.
    """

    url = "https://alphafold.ebi.ac.uk/files/AF-" + uniprot_code + "-F1-model_v3.pdb"

    fu.log("Downloading %s from: %s" % (uniprot_code, url), out_log, global_log)

    r = requests.get(url)
    if(r.status_code == 404):
        fu.log(classname + ': Incorrect Uniprot Code: %s' % (uniprot_code), out_log)
        raise SystemExit(classname + ': Incorrect Uniprot Code: %s' % (uniprot_code))

    return r.content.decode('utf-8')

def download_mmcif(pdb_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the mmcif file.
    """

    if api_id == 'mmb':
        url = "http://mmb.irbbarcelona.org/api/pdb/" + pdb_code + ".cif"
    elif api_id == 'pdb':
        url = "https://files.rcsb.org/download/" + pdb_code + ".cif"
    elif api_id == 'pdbe':
        url = "https://www.ebi.ac.uk/pdbe/entry-files/download/" + pdb_code + ".cif"

    fu.log("Downloading %s from: %s" % (pdb_code, url), out_log, global_log)
    return requests.get(url, verify=False).content.decode('utf-8')

def download_ligand(ligand_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the ligand file.
    """

    if api_id == 'mmb':
        url = "http://mmb.irbbarcelona.org/api/pdbMonomer/" + ligand_code.lower()
        text = requests.get(url, verify=False).content.decode('utf-8')
    elif api_id == 'pdbe':
        url="ftp://ftp.ebi.ac.uk/pub/databases/msd/pdbechem_v2/{0}/{1}/{1}_ideal.pdb".format(ligand_code.upper()[0], ligand_code.upper(), ligand_code.upper())
        text = urllib.request.urlopen(url).read().decode('utf-8')

    fu.log("Downloading %s from: %s" % (ligand_code, url), out_log, global_log)

    # removing useless empty lines at the end of the file
    text = os.linesep.join([s for s in text.splitlines() if s])
    
    return text

def download_fasta(pdb_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the fasta file.
    """

    if api_id == 'mmb':
        url = "http://mmb.irbbarcelona.org/api/pdb/" + pdb_code + ".fasta"
    elif api_id == 'pdb':
        url = "https://www.rcsb.org/fasta/entry/" + pdb_code
    elif api_id == 'pdbe':
        url = "https://www.ebi.ac.uk/pdbe/entry/pdb/" + pdb_code + "/fasta"

    fu.log("Downloading %s from: %s" % (pdb_code, url), out_log, global_log)
    return requests.get(url, verify=False).content.decode('utf-8')

def download_drugbank(drugbank_id, url="https://www.drugbank.ca/structures/small_molecule_drugs/%s.sdf?type=3d", out_log=None, global_log=None):
    """
    Returns:
        String: Content of the component file.
    """
    url = (url % drugbank_id)

    fu.log("Downloading %s from: %s" % (drugbank_id, url), out_log, global_log)

    text = requests.get(url, verify=False).content.decode('utf-8')
    
    return text

def download_binding_site(pdb_code, url="https://www.ebi.ac.uk/pdbe/api/pdb/entry/binding_sites/%s", out_log=None, global_log=None):
    """
    Returns:
        String: Content of the component file.
    """
    url = (url % pdb_code)

    fu.log("Getting binding sites from: %s" % (url), out_log, global_log)

    text = urllib.request.urlopen(url).read()
    json_obj = json.loads(text)
    json_string = json.dumps(json_obj, indent=4, sort_keys=True)
    #json_string = json.dumps(text, indent=4)
    
    return json_string

def download_ideal_sdf(ligand_code, api_id, out_log=None, global_log=None):
    """
    Returns:
        String: Content of the ideal sdf file.
    """

    if api_id == 'pdb':
        url = "https://files.rcsb.org/ligands/view/" + ligand_code.upper() + "_ideal.sdf"
        text = requests.get(url, verify=False).content.decode('utf-8')
    elif api_id == 'pdbe':
        url="ftp://ftp.ebi.ac.uk/pub/databases/msd/pdbechem_v2/{0}/{1}/{1}_ideal.sdf".format(ligand_code.upper()[0], ligand_code.upper(), ligand_code.upper())
        text = urllib.request.urlopen(url).read().decode('utf-8')

    fu.log("Downloading %s from: %s" % (ligand_code, url), out_log, global_log)
    
    return text

def download_str_info(pdb_code, url="http://mmb.irbbarcelona.org/api/pdb/%s.json", out_log=None, global_log=None):
    """
    Returns:
        String: Content of the JSON file.
    """
    url = (url % pdb_code)

    fu.log("Getting structure info from: %s" % (url), out_log, global_log)

    text = urllib.request.urlopen(url).read()
    json_obj = json.loads(text)
    json_string = json.dumps(json_obj, indent=4, sort_keys=True)
    #json_string = json.dumps(text, indent=4)
    
    return json_string

def write_pdb(pdb_string, output_pdb_path, filt=None, out_log=None, global_log=None):
    """ Writes and filters a PDB """
    fu.log("Writting pdb to: %s" % (output_pdb_path), out_log, global_log)
    with open(output_pdb_path, 'w') as output_pdb_file:
        if filt:
            fu.log("Filtering lines NOT starting with one of these words: %s" % str(filt), out_log, global_log)
            for line in pdb_string.splitlines(True):
                if line.strip().split()[0][0:6] in filt:
                    output_pdb_file.write(line)
        else:
            output_pdb_file.write(pdb_string)

def write_mmcif(mmcif_string, output_mmcif_path, out_log=None, global_log=None):
    """ Writes a mmcif """
    fu.log("Writting mmcif to: %s" % (output_mmcif_path), out_log, global_log)
    with open(output_mmcif_path, 'w') as output_mmcif_file:
        output_mmcif_file.write(mmcif_string)

def write_fasta(fasta_string, output_fasta_path, out_log=None, global_log=None):
    """ Writes a FASTA """
    fu.log("Writting FASTA to: %s" % (output_fasta_path), out_log, global_log)
    with open(output_fasta_path, 'w') as output_fasta_file:
        output_fasta_file.write(fasta_string)

def write_sdf(sdf_string, output_sdf_path, out_log=None, global_log=None):
    """ Writes a SDF """
    fu.log("Writting sdf to: %s" % (output_sdf_path), out_log, global_log)
    with open(output_sdf_path, 'w') as output_sdf_file:
        output_sdf_file.write(sdf_string)

def get_cluster_pdb_codes(pdb_code, cluster, out_log=None, global_log=None):
    """
    Returns:
        String list: The list of pdb_codes of the selected cluster.
    """
    url="http://mmb.irbbarcelona.org/api/pdb/"
    pdb_codes = set()

    url = url+pdb_code.lower()+'/clusters/cl-'+str(cluster)+".json"
    cluster_list = json.loads(requests.get(url, verify=False).content.decode('utf-8'))['clusterMembers']
    for elem in cluster_list:
        pdb_codes.add(elem['_id'].lower())

    if out_log:
        out_log.info('Cluster: '+str(cluster)+' of pdb_code: '+pdb_code+'\n List: '+str(pdb_codes))
    if global_log:
        global_log.info(fu.get_logs_prefix()+'Cluster: '+str(cluster)+' of pdb_code: '+pdb_code+'\n List: '+str(pdb_codes))

    return pdb_codes

def get_uniprot(pdb_code, url, out_log=None, global_log=None):
    """Returns the UNIPROT code corresponding to the `pdb_code`.

    Returns:
        str: UNIPROT code.
    """
    url_uniprot_id = (url+"/pdb/"+pdb_code.lower()+"/entry/uniprotRefs/_id")
    uniprot_id = requests.get(url_uniprot_id, verify=False).json()['uniprotRefs._id'][0]

    if out_log:
        out_log.info('PDB code: '+pdb_code+' correspond to uniprot id: '+uniprot_id)
    if global_log:
        global_log.info('PDB code: '+pdb_code+' correspond to uniprot id: '+uniprot_id)

    return uniprot_id

def get_variants(uniprot_id, url="http://mmb.irbbarcelona.org/api", out_log=None, global_log=None):
    """Returns the variants of the `uniprot_id` code.

    Returns:
        :obj:`list` of :obj:`str`: List of variants.
    """
    url_uniprot_mut = (url+"/uniprot/"+uniprot_id+"/entry/variants/vardata/mut/?varorig=humsavar")
    variants = requests.get(url_uniprot_mut, verify=False).json()['variants.vardata.mut']
    variants = variants if variants else []

    fu.log('Found: %d variants for uniprot id: %s' % (len(variants), uniprot_id), out_log, global_log)
    return variants if variants else []

def write_json(json_string, output_json_path, out_log=None, global_log=None):
    """ Writes a JSON """
    fu.log("Writting json to: %s" % (output_json_path), out_log, global_log)
    with open(output_json_path, 'w') as output_json_file:
        output_json_file.write(json_string)

def get_memprotmd_sim_list(out_log=None, global_log=None):
    """ Returns all available membrane-protein systems (simulations) from the MemProtMD DB using its REST API """

    fu.log('Getting all available membrane-protein systems (simulations) from the MemProtMD REST API', out_log, global_log)

    url = "http://memprotmd.bioch.ox.ac.uk/api/simulations/all"
    json_obj = requests.post(url).json()
    json_string = json.dumps(json_obj, indent=4)

    fu.log('Total number of simulations: %d' % (len(json_obj)), out_log, global_log)

    return json_string

def get_memprotmd_sim_search(collection_name, keyword, out_log=None, global_log=None):
    """ Performs advanced searches in the MemProtMD DB using its REST API and a given keyword """

    fu.log('Getting search results from the MemProtMD REST API. Collection name: %s, keyword: %s' % (collection_name, keyword), out_log, global_log)

    url = "http://memprotmd.bioch.ox.ac.uk/api/search/advanced"
    json_query = {
        "collectionName" : collection_name,
        "query" : {
            "keywords" : keyword
        },
        "projection" : {
            "simulations" : 1
        },
        "options" : {}
    }

    json_obj = requests.post(url, json=json_query).json()
    json_string = json.dumps(json_obj, indent=4)

    # get total number of simulation
    list_kw = []
    for sim_list in json_obj:
        for sim in sim_list['simulations']:
            list_kw.append(sim)

    fu.log('Total number of simulations: %d' % (len(list_kw)), out_log, global_log)

    return json_string

def get_memprotmd_sim(pdb_code, output_file, out_log=None, global_log=None):
    """ Gets a single simulation from MemProtMD DB """

    fu.log('Getting simulation file from pdb code %s' % (pdb_code), out_log, global_log)

    url = "http://memprotmd.bioch.ox.ac.uk/data/memprotmd/simulations/"+ pdb_code + "_default_dppc/files/run/at.zip"
    response = requests.get(url)

    open(output_file, 'wb').write(response.content)
    
    fu.log("Saving output %s file" % (output_file), out_log, global_log)

def check_mandatory_property(property, name, out_log, classname):
    """ Checks mandatory properties """

    if not property:
        fu.log(classname + ': Unexisting %s property, exiting' % name, out_log)
        raise SystemExit(classname + ': Unexisting %s property' % name)
    return property

def check_uniprot_code(code, out_log, classname):
    """ Checks uniprot code """

    pattern = re.compile((r"[OPQ][0-9][A-Z0-9]{3}[0-9]|[A-NR-Z][0-9]([A-Z][A-Z0-9]{2}[0-9]){1,2}"))

    if not pattern.match(code):
        fu.log(classname + ': Incorrect uniprot code for %s' % code, out_log)
        raise SystemExit(classname + ': Incorrect uniprot code for %s' % code)

    return True


#!/usr/bin/env python

"""PdbClusterZip Module"""
import os
import argparse
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_io.api.common import *


class PdbClusterZip(BiobbObject):
    """
    | biobb_io PdbClusterZip
    | This class is a wrapper for downloading a PDB cluster from the Protein Data Bank.
    | Wrapper for the `Protein Data Bank in Europe <https://www.ebi.ac.uk/pdbe/>`_, the `Protein Data Bank <https://www.rcsb.org/>`_ and the `MMB PDB mirror <http://mmb.irbbarcelona.org/api/>`_ for downloading a PDB cluster.

    Args:
        output_pdb_zip_path (str): Path to the ZIP file containing the output PDB files. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_pdb_cluster.zip>`_. Accepted formats: zip (edam:format_3987).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB code.
            * **filter** (*str*) - (["ATOM", "MODEL", "ENDMDL"]) Array of groups to be kept. If value is None or False no filter will be applied. All the possible values are defined in the official PDB specification (http://www.wwpdb.org/documentation/file-format-content/format33/v3.3.html)
            * **cluster** (*int*) - (90) Sequence Similarity Cutoff. Values: 50 (structures having less than 50% sequence identity to each other), 70 (structures having less than 70% sequence identity to each other), 90 (structures having less than 90% sequence identity to each other), 95 (structures having less than 95% sequence identity to each other).
            * **api_id** (*str*) - ("pdbe") Identifier of the PDB REST API from which the PDB structure will be downloaded. Values: pdbe (`PDB in Europe REST API <https://www.ebi.ac.uk/pdbe/pdbe-rest-api>`_), pdb (`RCSB PDB REST API <https://data.rcsb.org/>`_), mmb (`MMB PDB mirror API <http://mmb.irbbarcelona.org/api/>`_).
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.pdb_cluster_zip import pdb_cluster_zip
            prop = { 
                'pdb_code': '2VGB', 
                'filter': ['ATOM', 'MODEL', 'ENDMDL'], 
                'cluster': 90, 
                'api_id': 'pdbe' 
            }
            pdb_cluster_zip(output_pdb_zip_path='/path/to/newStructures.zip', 
                            properties=prop)

    Info:
        * wrapped_software:
            * name: Protein Data Bank
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_pdb_zip_path, 
                properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = { 
            "out": { "output_pdb_zip_path": output_pdb_zip_path } 
        }

        # Properties specific for BB
        self.api_id = properties.get('api_id', 'pdbe')
        self.pdb_code = properties.get('pdb_code', None)
        self.filter = properties.get('filter', ['ATOM', 'MODEL', 'ENDMDL'])
        self.cluster = properties.get('cluster', 90)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """ Checks all the input/output paths and parameters """
        self.output_pdb_zip_path = check_output_path(self.io_dict["out"]["output_pdb_zip_path"], "output_pdb_zip_path", False, out_log, self.__class__.__name__)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`PdbClusterZip <api.pdb_cluster_zip.PdbClusterZip>` api.pdb_cluster_zip.PdbClusterZip object."""
        
        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart(): return 0
        #self.stage_files()

        check_mandatory_property(self.pdb_code, 'pdb_code', self.out_log, self.__class__.__name__)

        self.pdb_code = self.pdb_code.strip().lower()

        file_list = []
        #Downloading PDB_files
        pdb_code_list = get_cluster_pdb_codes(pdb_code=self.pdb_code, cluster=self.cluster, out_log=self.out_log, global_log=self.global_log)
        unique_dir = fu.create_unique_dir()
        for pdb_code in pdb_code_list:
            pdb_file = os.path.join(unique_dir, pdb_code+".pdb")
            pdb_string = download_pdb(pdb_code=pdb_code, api_id=self.api_id, out_log=self.out_log, global_log=self.global_log)
            write_pdb(pdb_string, pdb_file, self.filter, self.out_log, self.global_log)
            file_list.append(os.path.abspath(pdb_file))

        #Zipping files
        fu.log("Zipping the pdb files to: %s" % self.output_pdb_zip_path)
        fu.zip_list(self.output_pdb_zip_path, file_list, out_log=self.out_log)

        self.tmp_files.extend([
            self.stage_io_dict.get("unique_dir"),
            unique_dir
        ])
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0

def pdb_cluster_zip(output_pdb_zip_path: str, properties: dict = None, **kwargs) -> int:
    """Execute the :class:`PdbClusterZip <api.pdb_cluster_zip.PdbClusterZip>` class and
    execute the :meth:`launch() <api.pdb_cluster_zip.PdbClusterZip.launch>` method."""

    return PdbClusterZip(output_pdb_zip_path=output_pdb_zip_path,
                        properties=properties, **kwargs).launch()

def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="Wrapper for the Protein Data Bank in Europe (https://www.ebi.ac.uk/pdbe/), the Protein Data Bank (https://www.rcsb.org/) and the MMB PDB mirror (http://mmb.irbbarcelona.org/api/) for downloading a PDB cluster.", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('-c', '--config', required=False, help="This file can be a YAML file, JSON file or JSON string")

    #Specific args of each building block
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('-o','--output_pdb_zip_path', required=True, help="Path to the ZIP or PDB file containing the output PDB files. Accepted formats: pdb, zip.")

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    #Specific call of each building block
    pdb_cluster_zip(output_pdb_zip_path=args.output_pdb_zip_path, 
                properties=properties)

if __name__ == '__main__':
    main()



name = "api"
__all__ = ["drugbank", "ligand", "pdb", "alphafold", "pdb_cluster_zip", "pdb_variants", "memprotmd_sim_list", "memprotmd_sim_search", "memprotmd_sim", "api_binding_site","canonical_fasta", "mmcif", "ideal_sdf", "structure_info"]


#!/usr/bin/env python

"""PdbVariants Module"""
import re
import argparse
import requests
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_io.api.common import *


class PdbVariants(BiobbObject):
    """
    | biobb_io PdbVariants
    | This class creates a text file containing a list of all the variants mapped to a PDB code from the corresponding UNIPROT entries.
    | Wrapper for the `UNIPROT <http://www.uniprot.org/>`_ mirror of the `MMB group REST API <http://mmb.irbbarcelona.org/api/>`_ for creating a list of all the variants mapped to a PDB code from the corresponding UNIPROT entries.

    Args:
        output_mutations_list_txt (str): Path to the TXT file containing an ASCII comma separated values of the mutations. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_pdb_variants.txt>`_. Accepted formats: txt (edam:format_2330).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB four letter code.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.

    Examples:
            This is a use example of how to use the PdbVariants module from Python

            from biobb_io.api.pdb_variants import pdb_variants
            prop = { 
                'pdb_code': '2VGB' 
            }
            pdb_variants(output_mutations_list_txt='/path/to/newMutationsList.txt', 
                        properties=prop)

    Info:
        * wrapped_software:
            * name: UNIPROT
            * license: Creative Commons
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_mutations_list_txt, 
                properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = { 
            "out": { "output_mutations_list_txt": output_mutations_list_txt } 
        }

        # Properties specific for BB
        self.pdb_code = properties.get('pdb_code', None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, err_log):
        """ Checks all the input/output paths and parameters """
        self.output_mutations_list_txt = check_output_path(self.io_dict["out"]["output_mutations_list_txt"], "output_mutations_list_txt", False, out_log, self.__class__.__name__)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`PdbVariants <api.pdb_variants.PdbVariants>` api.pdb_variants.PdbVariants object."""
        
        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart(): return 0
        #self.stage_files()

        check_mandatory_property(self.pdb_code, 'pdb_code', self.out_log, self.__class__.__name__)

        self.pdb_code = self.pdb_code.strip().lower()

        url = 'http://mmb.irbbarcelona.org/api'
        uniprot_id = get_uniprot(self.pdb_code, url, self.out_log, self.global_log)
        url_mapPDBRes = (url+"/uniprot/"+uniprot_id+"/mapPDBRes?pdbId="+self.pdb_code)
        pattern = re.compile((r"p.(?P<wt>[a-zA-Z]{3})(?P<resnum>\d+)(?P<mt>[a-zA-Z]{3})"))

        fu.log('Fetching variants for uniprot_id: %s and pdb_code: %s' % (uniprot_id, self.pdb_code), self.out_log, self.global_log)
        unfiltered_dic = requests.get(url_mapPDBRes, verify=False).json()
        if not unfiltered_dic:
            fu.log("No mutation found", self.out_log, self.global_log)
            return None

        mapdic = requests.get(url_mapPDBRes, verify=False).json()
        mutations = []
        uniprot_var_list = get_variants(uniprot_id, url, self.out_log, self.global_log)
        for var in uniprot_var_list:
            uni_mut = pattern.match(var).groupdict()
            for k in mapdic.keys():
                for fragment in mapdic[k]:
                    if int(fragment['unp_start']) <= int(uni_mut['resnum']) <= int(fragment['unp_end']):
                        resnum = int(uni_mut['resnum']) + int(fragment['pdb_start']) - int(fragment['unp_start'])
                        mutations.append(k[-1]+'.'+uni_mut['wt']+str(resnum)+uni_mut['mt'])

        fu.log('Found %d mutations mapped to PDB: %s' % (len(mutations), self.pdb_code), self.out_log, self.global_log)
        fu.log('Writting mutations to: %s' % self.output_mutations_list_txt, self.out_log, self.global_log)

        with open(self.output_mutations_list_txt, 'w') as mut_file:
            mutations.sort()
            mut_file.write(",".join(mutations))

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0

def pdb_variants(output_mutations_list_txt: str, properties: dict = None, **kwargs) -> int:
    """Execute the :class:`PdbVariants <api.pdb_variants.PdbVariants>` class and
    execute the :meth:`launch() <api.pdb_variants.PdbVariants.launch>` method."""

    return PdbVariants(output_mutations_list_txt=output_mutations_list_txt,
                        properties=properties, **kwargs).launch()

def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="Wrapper for the UNIPROT (http://www.uniprot.org/) mirror of the MMB group REST API (http://mmb.irbbarcelona.org/api/) for creating a list of all the variants mapped to a PDB code from the corresponding UNIPROT entries.", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('-c', '--config', required=False, help="This file can be a YAML file, JSON file or JSON string")

    #Specific args of each building block
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('-o', '--output_mutations_list_txt', required=True, help="Path to the TXT file containing an ASCII comma separated values of the mutations. Accepted formats: txt.")

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    #Specific call of each building block
    pdb_variants(output_mutations_list_txt=args.output_mutations_list_txt, 
                properties=properties)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""Module containing the Drugbank class and the command line interface."""
import argparse
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_io.api.common import *


class Drugbank(BiobbObject):
    """
    | biobb_io Drugbank
    | This class is a wrapper for the `Drugbank <https://www.drugbank.ca/>`_ REST API.
    | Download a single component in SDF format from the `Drugbank <https://www.drugbank.ca/>`_ REST API.

    Args:
        output_sdf_path (str): Path to the output SDF component file. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_drugbank.sdf>`_. Accepted formats: sdf (edam:format_3814).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **drugbank_id** (*str*) - (None) Drugbank component id.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.drugbank import drugbank
            prop = { 
                'drugbank_id': 'DB00530' 
            }
            drugbank(output_sdf_path='/path/to/newComponent.sdf', 
                    properties=prop)

    Info:
        * wrapped_software:
            * name: Drugbank
            * license: Creative Commons
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_sdf_path, 
                properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = { 
            "out": { "output_sdf_path": output_sdf_path } 
        }

        # Properties specific for BB
        self.drugbank_id = properties.get('drugbank_id', None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()
        
    def check_data_params(self, out_log, err_log):
        """ Checks all the input/output paths and parameters """
        self.output_sdf_path = check_output_path(self.io_dict["out"]["output_sdf_path"], "output_sdf_path", False, out_log, self.__class__.__name__)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`Drugbank <api.drugbank.Drugbank>` api.drugbank.Drugbank object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart(): return 0
        #self.stage_files()

        check_mandatory_property(self.drugbank_id, 'drugbank_id', self.out_log, self.__class__.__name__)

        self.drugbank_id = self.drugbank_id.strip().lower()
        url = "https://www.drugbank.ca/structures/small_molecule_drugs/%s.sdf?type=3d"

        #Downloading SDF file
        sdf_string = download_drugbank(self.drugbank_id, url, self.out_log, self.global_log)
        write_sdf(sdf_string, self.output_sdf_path, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0

def drugbank(output_sdf_path: str, properties: dict = None, **kwargs) -> int:
    """Execute the :class:`Drugbank <api.drugbank.Drugbank>` class and
    execute the :meth:`launch() <api.drugbank.Drugbank.launch>` method."""

    return Drugbank(output_sdf_path=output_sdf_path,
                    properties=properties, **kwargs).launch()

def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="Download a component in SDF format from the Drugbank (https://www.drugbank.ca/).", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('-c', '--config', required=False, help="This file can be a YAML file, JSON file or JSON string")

    #Specific args of each building block
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('-o', '--output_sdf_path', required=True, help="Path to the output SDF component file. Accepted formats: sdf.")

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    #Specific call of each building block
    drugbank(output_sdf_path=args.output_sdf_path, 
            properties=properties)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""Module containing the ApiBindingSite class and the command line interface."""
import argparse
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import  settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_io.api.common import *


class ApiBindingSite(BiobbObject):
    """
    | biobb_io ApiBindingSite
    | This class is a wrapper for the `PDBe REST API <https://www.ebi.ac.uk/pdbe/api/doc/#pdb_apidiv_call_16_calltitle>`_ Binding Sites endpoint.
    | This call provides details on binding sites in the entry as per STRUCT_SITE records in PDB files, such as ligand, residues in the site, description of the site, etc.

    Args:
        output_json_path (str): Path to the JSON file with the binding sites for the requested structure. File type: output. `Sample file <https://github.com/bioexcel/biobb_io/raw/master/biobb_io/test/reference/api/output_binding_site.json>`_. Accepted formats: json (edam:format_3464).
        properties (dic - Python dictionary object containing the tool parameters, not input/output files):
            * **pdb_code** (*str*) - (None) RSCB PDB code.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_io.api.api_binding_site import api_binding_site
            prop = { 
                'pdb_code': '4i23' 
            }
            api_binding_site(output_json_path='/path/to/newBindingSites.json', 
                    properties=prop)

    Info:
        * wrapped_software:
            * name: PDBe REST API
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, output_json_path, 
                properties=None, **kwargs) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = { 
            "out": { "output_json_path": output_json_path } 
        }

        # Properties specific for BB
        self.pdb_code = properties.get('pdb_code', None)
        self.properties = properties

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()
        
    def check_data_params(self, out_log, err_log):
        """ Checks all the input/output paths and parameters """
        self.output_json_path = check_output_path(self.io_dict["out"]["output_json_path"], "output_json_path", False, out_log, self.__class__.__name__)

    @launchlogger
    def launch(self) -> int:
        """Execute the :class:`ApiBindingSite <api.api_binding_site.ApiBindingSite>` api.api_binding_site.ApiBindingSite object."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart(): return 0
        #self.stage_files()

        check_mandatory_property(self.pdb_code, 'pdb_code', self.out_log, self.__class__.__name__)

        self.pdb_code = self.pdb_code.strip().lower()
        url = "https://www.ebi.ac.uk/pdbe/api/pdb/entry/binding_sites/%s"

        # get JSON object
        json_string = download_binding_site(self.pdb_code, url, self.out_log, self.global_log)

        # get number of binding sites
        fu.log('%d binding sites found' % (len(json.loads(json_string)[self.pdb_code])), self.out_log, self.global_log)

        # write JSON file
        write_json(json_string, self.output_json_path, self.out_log, self.global_log)

        self.check_arguments(output_files_created=True, raise_exception=False)

        return 0

def api_binding_site(output_json_path: str, properties: dict = None, **kwargs) -> int:
    """Execute the :class:`ApiBindingSite <api.api_binding_site.ApiBindingSite>` class and
    execute the :meth:`launch() <api.api_binding_site.ApiBindingSite.launch>` method."""

    return ApiBindingSite(output_json_path=output_json_path,
                    properties=properties, **kwargs).launch()

def main():
    """Command line execution of this building block. Please check the command line documentation."""
    parser = argparse.ArgumentParser(description="This class is a wrapper for the PDBe REST API Binding Sites endpoint", formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('-c', '--config', required=False, help="This file can be a YAML file, JSON file or JSON string")

    #Specific args of each building block
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('-o', '--output_json_path', required=True, help="Path to the JSON file with the binding sites for the requested structure. Accepted formats: json.")

    args = parser.parse_args()
    config = args.config if args.config else None
    properties = settings.ConfReader(config=config).get_prop_dic()

    #Specific call of each building block
    api_binding_site(output_json_path=args.output_json_path, 
            properties=properties)

if __name__ == '__main__':
    main()
